
#include "morsetree.h"
using namespace std;

namespace AlgorithMan
{   
    
    MorseTree::MorseTree()
    {
        root = new MorseTreeNode(NULL, ' ');
        root->content = ' ';
        
        Insert(".-",'A');
        Insert("-...",'B');
        Insert("-.-.",'C');
        Insert("-..",'D');
        Insert(".",'E');
        Insert("..-.",'F');
        Insert("--.",'G');
        Insert("....",'H');
        Insert("..",'I');
        Insert(".---",'J');
        Insert("-.-",'K');
        Insert(".-..",'L');
        Insert("--",'M');
        Insert("-.",'N');
        Insert("---",'O');
        Insert(".--.",'P');
        Insert("--.-",'Q');
        Insert(".-.",'R');
        Insert("...",'S');
        Insert("-",'T');
        Insert("..-",'U');
        Insert("...-",'V');
        Insert(".--",'W');
        Insert("-..-",'X');
        Insert("-.--",'Y');
        Insert("--..",'Z');

        Insert("-----",'0');
        Insert(".----",'1');
        Insert("..---",'2');
        Insert("...--",'3');
        Insert("....-",'4');
        Insert(".....",'5');
        Insert("-....",'6');
        Insert("--...",'7');
        Insert("---..",'8');
        Insert("----.",'9');
        
        Insert(".-.-.-",'.');
        Insert("--..--",',');
        Insert("---...",':');
        Insert("-.-.-.",';');
        Insert("..--..",'?');
        Insert("-....-",'-');
        Insert("..--.-",'_');
        Insert("-.--.",'(');
        Insert("-.--.-",')');
        Insert(".----.",'\'');
        Insert("-...-",'=');
        Insert(".-.-.",'+');
        Insert("-..-.",'/');
        Insert(".--.-.",'@');
    }
    
    
    
    MorseTree::~MorseTree()
    {
        if(root != NULL)
            delete root;
    }
    
    
    
    void MorseTree::Insert(string code, char character)
    {
        MorseTreeNode* n = root;
        for(string::iterator i = code.begin(); i != code.end(); i++)
        {
            if(*i == '.')
            {
                if(n->short_succ == NULL)
                    n->short_succ = new MorseTreeNode(n, '.');
                n = n->short_succ;
            }
            else if(*i == '-')
            {
                if(n->long_succ == NULL)
                    n->long_succ = new MorseTreeNode(n, '-');
                n = n->long_succ;
            }
            else
                return;
        }
        n->content = character;
        registered[character] = n;
    }
    
    
    
    string MorseTree::ToMorseCode(MorseTreeNode* n)
    {
        if(n->parent == NULL)
            return "";
        return ToMorseCode(n->parent) + n->codepart;
    }
    
    
    
    string MorseTree::ToMorseCode(string text)
    {
        string result;
        for(string::iterator i = text.begin(); i != text.end(); i++)
        {
            if(*i == ' ')
                result += "  ";
            else
            {
                char c = *i;
                if('a' <= c && c <= 'z')
                    c = c - 'a' + 'A';
                map<char, MorseTreeNode*>::iterator n = registered.find(c);
                if(n != registered.end())
                    result += ToMorseCode(n->second) + " ";
            }
        }
        return result;
    }

    
    
    string MorseTree::FromMorseCode(string text)
    {
        string result;
        MorseTreeNode* n = root;;
        int countspaces = 0;
        for(string::iterator i = text.begin(); i != text.end(); i++)
        {
            if(*i != ' ')
                countspaces = 0;
            
            switch(*i)
            {
                case '.':
                    n = n->short_succ == NULL ? root : n->short_succ;
                    break;
                case '-':
                    n = n->long_succ == NULL ? root : n->long_succ;
                    break;
                case ' ':
                    if(++countspaces == 3)
                    {
                        countspaces = 0;
                    }
                    else
                    {
                        result += n->content;
                        n = root;
                    }
                    break;
            }
        }
        return result;
    }
    
    
    
    MorseTreeNode::MorseTreeNode(MorseTreeNode* parent, char codepart)
    {
        this->parent = parent;
        this->codepart = codepart;
        long_succ = NULL;
        short_succ = NULL;
    }
    
    
    
    MorseTreeNode::~MorseTreeNode()
    {
        if(long_succ != NULL)
            delete long_succ;
        if(short_succ != NULL)
            delete short_succ;
    }
    
}
